class CreateUserRequest:
    def __init__(self):
        self.name = ''
        self.email = ''
        self.role = ''


class User:
    def __init__(self):
        self.id = ''
        self.email = ''
        self.role = ''
        self.name = ''
        self.createdAt = ''
        self.updatedAt = ''
        self.image = ''



from requests.auth import HTTPBasicAuth
from collections import namedtuple
import jsonpickle
import requests
import json


"""
    HTTP Transport via the library 'requests'
    - JSON Serialization via 'jsonpickle'
    - JSON Deserialization via general mapping
    - Authentication with the Domo API is handled automatically (OAuth2)
"""


class DomoAPITransport:
    def __init__(self, client_id, client_secret, api_host, use_https, logger):
        self.apiHost = self._build_apihost(api_host, use_https)
        self.clientId = client_id
        self.clientSecret = client_secret
        self.access_token = ''
        self.logger = logger

    @staticmethod
    def _build_apihost(host, use_https):
        if use_https:
            host = 'https://' + host
        else:
            host = 'http://' + host
        return host

    def get(self, url, params):
        return self.request(url, HTTPMethod.GET, self._headers_receive_json(), params, {})

    def get_csv(self, url, params):
        return self.request(url, HTTPMethod.GET, self._headers_receive_csv(), params, {})

    def post(self, url, body, params):
        return self.request(url, HTTPMethod.POST, self._headers_send_json(), params, self._obj_to_json(body))

    def put(self, url, body):
        return self.request(url, HTTPMethod.PUT, self._headers_send_json(), {}, self._obj_to_json(body))

    def put_csv(self, url, body):
        return self.request(url, HTTPMethod.PUT, self._headers_send_csv(), {}, body)

    def patch(self, url, body):
        return self.request(url, HTTPMethod.PATCH, self._headers_send_json(), {}, self._obj_to_json(body))

    def delete(self, url):
        return self.request(url, HTTPMethod.DELETE, self._headers_receive_json(), {}, {})

    def request(self, url, method, headers, params, body):
        url = self.build_url(url)
        self.logger.info(method + " " + url + "" + str(body))
        return requests.request(method=method, url=url, headers=headers, data=body, params=params)

    def build_url(self, url):
        return self.apiHost + url

    @staticmethod
    def _obj_to_json(obj):
        return jsonpickle.encode(obj, unpicklable=False)

    def json_to_obj(self, blob):
        return json.loads(blob, object_hook=lambda json_dict: self._filter_reserved_words(json_dict))

    @staticmethod
    def _filter_reserved_words(json_dict):
        keys = list(json_dict.keys())
        i = 0
        for key in keys:
            if key == 'not':
                keys[i] = 'NOT'
            i += 1
        return namedtuple('x', keys)(*json_dict.values())

    def print_json(self, message, json_text):
        self.logger.info(message + ": " + json.dumps(json_text, indent=4))

    def check_renew_token(self):
        if self._token_expired():
            self.access_token = self._renew_access_token()

    def _renew_access_token(self):
        self.logger.info("Renewing Access Token")
        url = self.apiHost + '/oauth/token?grant_type=client_credentials&scope=data user'
        response = requests.post(url=url, auth=HTTPBasicAuth(self.clientId, self.clientSecret))
        if response.status_code == requests.codes.ok:
            return response.json()['access_token']
        else:
            raise Exception("Error retrieving a Domo API Access Token: " + str(response.json()))

    def _token_expired(self):
        token_expired = False
        url = self.apiHost + '/v1/users/'
        response = requests.get(url=url, headers=self._headers_receive_json())
        if response.status_code == requests.codes.unauthorized:
            token_expired = True
        return token_expired

    def _headers_receive_json(self):
        return {
            'Authorization': 'bearer ' + self.access_token,
            'Accept': 'application/json'
        }

    def _headers_send_json(self):
        headers = self._headers_receive_json()
        headers['Content-Type'] = 'application/json'
        return headers

    def _headers_send_csv(self):
        headers = self._headers_receive_json()
        headers['Content-Type'] = 'text/csv'
        return headers

    def _headers_receive_csv(self):
        headers = self._headers_receive_json()
        headers['Accept'] = 'text/csv'
        return headers


class HTTPMethod:
    GET = 'GET'
    POST = 'POST'
    PUT = 'PUT'
    PATCH = 'PATCH'
    DELETE = 'DELETE'



from pydomo.DomoAPIClient import DomoAPIClient
from pydomo.Transport import HTTPMethod
import requests

"""
    DataSets
    - Programmatically manage Domo DataSets
    - Use DataSets for fairly static data sources that only require occasional updates via data replacement
    - Use Streams if your data source is massive, constantly changing, or rapidly growing
    - Docs: https://developer.domo.com/docs/data-apis/data
"""


class DataSetClient(DomoAPIClient):
    def __init__(self, transport, logger):
        super(DataSetClient, self).__init__(transport, logger)
        self.urlBase = '/v1/datasets/'
        self.dataSetDesc = "DataSet"
        self.PDPDesc = "Personalized Data Policy (PDP)"

    """
        Create a DataSet
    """
    def create(self, dataset_request):
        return self._create(self.urlBase, dataset_request, {}, self.dataSetDesc)

    """
        Get a DataSet
    """
    def get(self, dataset_id):
        return self._get(self._base(dataset_id), self.dataSetDesc)

    """
        List DataSets
        - The 'limit' max is 50; use offset pagination to retrieve more DataSets
    """
    def list(self, sort, limit, offset):
        params = {
            'sort': str(sort),
            'limit': str(limit),
            'offset': str(offset),
        }
        return self._list(self.urlBase, params, self.dataSetDesc)

    """
        Update a DataSet
    """
    def update(self, dataset_id, dataset_update):
        return self._update(self._base(dataset_id), HTTPMethod.PUT, requests.codes.ok, dataset_update, self.dataSetDesc)

    """
        Import data from a CSV string
    """
    def data_import(self, dataset_id, csv):
        url = self._base(dataset_id) + '/data'
        return self._upload_csv(url, requests.codes.no_content, csv, self.dataSetDesc)

    """
        Import data from a CSV file
    """
    def data_import_from_file(self, dataset_id, filepath):
        with open(filepath, 'r') as csvfile:
            # passing an open file to the requests library invokes http streaming (uses minimal system memory)
            self.data_import(dataset_id, csvfile)

    """
        Export data to a CSV string
    """
    def data_export(self, dataset_id, include_csv_header):
        url = self._base(dataset_id) + '/data'
        params = {
            'includeHeader': str(include_csv_header),
            'fileName': 'foo.csv'
        }
        response = self.transport.get_csv(url=url, params=params)
        if response.status_code == requests.codes.ok:
            return response.text
        else:
            raise Exception("Error downloading data from DataSet: " + str(response.json()))

    """
        Export data to a CSV file, and return the readable/writable object file
    """
    def data_export_to_file(self, dataset_id, file_path, include_csv_header):
        file_path = str(file_path)
        if '.csv' not in file_path:
            file_path += '.csv'
        csv_str = self.data_export(dataset_id, include_csv_header)
        with open(file_path, "w") as csv_file:
            csv_file.write(csv_str)
        return open(file_path, "r+")  # return the file object as readable and writable

    """
        Delete a DataSet
    """
    def delete(self, dataset_id):
        return self._delete(self._base(dataset_id), self.dataSetDesc)

    """
        Create a Personalized Data Policy (PDP)
    """
    def create_pdp(self, dataset_id, pdp_request):
        url = self._base(dataset_id) + '/policies'
        return self._create(url, pdp_request, {}, self.PDPDesc)

    """
        Get a specific Personalized Data Policy (PDP) for a given DataSet
    """
    def get_pdp(self, dataset_id, policy_id):
        return self._get(self._base(dataset_id), self.PDPDesc)

    """
        List all Personalized Data Policies (PDPs) for a given DataSet
    """
    def list_pdps(self, dataset_id):
        url = self._base(dataset_id) + '/policies'
        return self._list(url, {}, self.dataSetDesc)

    """
        Update a specific Personalized Data Policy (PDP) for a given DataSet
    """
    def update_pdp(self, dataset_id, policy_id, policy_update):
        url = self._base(dataset_id) + '/policies/' + str(policy_id)
        return self._update(url, HTTPMethod.PUT, requests.codes.ok, policy_update, self.PDPDesc)

    """
        Delete a specific Personalized Data Policy (PDPs) for a given DataSet
    """
    def delete_pdp(self, dataset_id, policy_id):
        url = self._base(dataset_id) + '/policies/' + str(policy_id)
        return self._delete(url, self.PDPDesc)



class Column:
    def __init__(self, column_type, name):
        self.type = column_type
        self.name = name


class ColumnType:
    STRING = 'STRING'
    DECIMAL = 'DECIMAL'
    LONG = 'LONG'
    DOUBLE = 'DOUBLE'
    DATE = 'DATE'
    DATETIME = 'DATETIME'


class DataSetRequest:
    def __init__(self):
        self.name = ''
        self.description = ''
        self.schema = ''


class DataSet:
    def __init__(self):
        self.id = 0
        self.name = ''
        self.description = ''
        self.rows = 0
        self.columns = 0
        self.schema = Schema([])
        self.owner = Owner()
        self.createdAt = ''
        self.updatedAt = ''


class DataSetAndPDP(DataSet):
    def __init__(self):
        super().__init__()
        self.pdpEnabled = False
        self.policies = []


class DataSetListResult:
    def __init__(self):
        self.id = 0
        self.name = ''
        self.description = ''
        self.rows = 0
        self.columns = 0
        self.owner = Owner()
        self.dataCurrentAt = ''
        self.createdAt = ''
        self.updatedAt = ''
        self.pdpEnabled = False


class FilterOperator:
    EQUALS = 'EQUALS'
    LIKE = 'LIKE'
    GREATER_THAN = 'GREATER_THAN'
    LESS_THAN = 'LESS_THAN'
    GREATER_THAN_EQUAL = 'GREATER_THAN_EQUAL'
    LESS_THAN_EQUAL = 'LESS_THAN_EQUAL'
    BETWEEN = 'BETWEEN'
    BEGINS_WITH = 'BEGINS_WITH'
    ENDS_WITH = 'ENDS_WITH'
    CONTAINS = 'CONTAINS'


class Owner:
    def __init__(self):
        self.id = 0
        self.name = ''


class Policy:
    def __init__(self):
        self.id = 0
        self.type = ''
        self.name = ''
        self.filters = []
        self.users = []
        self.groups = []


class PolicyType:
    USER = 'user'
    SYSTEM = 'system'


class PolicyFilter:
    def __init__(self):
        self.column = ''
        self.values = []
        self.operator = ''
        self.NOT = False


class Schema:
    def __init__(self, columns):
        self.columns = columns



class Group:
    def __init__(self):
        self.id = 0
        self.name = ''
        self.active = True
        self.creatorId = ''
        self.default = False


class CreateGroupRequest:
    def __init__(self):
        self.name = ''
        self.active = True
        self.default = False
        self.memberCount = 0



from .DataSetModel import Column, ColumnType, DataSetRequest, DataSet, DataSetAndPDP, DataSetListResult
from .DataSetModel import FilterOperator, Owner, Policy, PolicyType, PolicyFilter, Schema
from .DataSetClient import DataSetClient



from pydomo.Transport import HTTPMethod
import requests

"""
    DomoAPIClient
    - Each API client class inherits from this superclass
"""


class DomoAPIClient:
    def __init__(self, transport, logger):
        self.transport = transport
        self.logger = logger

    def _create(self, url, request, params, obj_desc):
        self.transport.check_renew_token()
        response = self.transport.post(url=url, params=params, body=request)
        if response.status_code == requests.codes.created or response.status_code == requests.codes.ok:
            obj = self.transport.json_to_obj(response.text)
            self.transport.print_json("Created", str(response.json()))
            return obj
        else:
            raise Exception("Error creating " + str(obj_desc) + ": " + str(response.json()))

    def _get(self, url, obj_desc):
        self.transport.check_renew_token()
        response = self.transport.get(url=url, params={})
        if response.status_code == requests.codes.ok:
            obj = self.transport.json_to_obj(response.text)
            return obj
        else:
            raise Exception("Error retrieving " + str(obj_desc) + ": " + str(response.json()))

    def _update(self, url, method, success_code, obj_update, obj_desc):
        self.transport.check_renew_token()
        response = ''
        if method == HTTPMethod.PUT:
            response = self.transport.put(url, obj_update)
        elif method == HTTPMethod.PATCH:
            response = self.transport.patch(url, obj_update)
        if response.status_code == success_code:
            if str(response.text) == '':
                return
            else:
                obj = self.transport.json_to_obj(response.text)
                self.transport.print_json("Updated", str(response.json()))
                return obj
        else:
            raise Exception("Error updating " + str(obj_desc) + ": " + str(response.json()))

    def _list(self, url, params, obj_desc):
        self.transport.check_renew_token()
        response = self.transport.get(url=url, params=params)
        if response.status_code == requests.codes.ok:
            obj_list = self.transport.json_to_obj(response.text)
            return obj_list
        else:
            raise Exception(obj_desc + " Error: " + str(response.json()))

    def _delete(self, url, obj_desc):
        self.transport.check_renew_token()
        response = self.transport.delete(url=url)
        if response.status_code == requests.codes.no_content:
            return
        else:
            raise Exception("Error deleting " + str(obj_desc) + ": " + str(response.json()))

    def _upload_csv(self, url, success_code, csv, obj_desc):
        self.transport.check_renew_token()
        response = self.transport.put_csv(url=url, body=csv)
        if response.status_code == success_code:
            if str(response.text) == '':
                return
            else:
                return self.transport.json_to_obj(response.text)
        else:
            raise Exception("Error uploading " + str(obj_desc) + ": " + str(response.json()))

    def _base(self, obj_id):
        return self.urlBase + str(obj_id)



from distutils.core import setup


setup(
    name='pydomo',
    version='0.1.2',
    description='The official python3 Domo API SDK - Domo, Inc.',
    author='Bobby Swingler',
    author_email='bobby.swingler@domo.com',
    url='https://github.com/domoinc/domo-python-sdk',
    keywords=['domo', 'api', 'sdk'],
    license='MIT',
    packages=['pydomo',
              'pydomo.datasets',
              'pydomo.groups',
              'pydomo.streams',
              'pydomo.users'],
    requires=[
        'requests',
        'jsonpickle'
    ]
)



from pydomo.datasets.DataSetModel import DataSet


class Stream:
    def __init__(self):
        self.id = 0
        self.dataSet = DataSet()
        self.updateMethod = ''
        self.createdAt = ''
        self.modifiedAt = ''


class CreateStreamRequest:
    def __init__(self, data_set_request, update_method):
        self.dataSet = data_set_request
        self.updateMethod = update_method


class Execution:
    def __init__(self):
        self.id = 0
        self.startedAt = ''
        self.currentState = ''
        self.createdAt = ''
        self.modifiedAt = ''


class UpdateMethod:
    APPEND = 'APPEND'
    REPLACE = 'REPLACE'
